@@ -1,8 +1,18 @@
 
 
+# forced=NULL; hierarchical=NULL;
+# plot_type = 'residuals';
+# CI_level = 95;
+# MCMC = FALSE;
+# Nsamples = 10000;
+# verbose=TRUE
+# 
+# data=data_Green_Salkind_2014; DV='injury';
+#                forced = c('quads','gluts','abdoms','arms','grip')
+
+
 
 OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
-                            COVARS = NULL,
                             plot_type = 'residuals', 
                             CI_level = 95,
                             MCMC = FALSE,
@@ -26,7 +36,6 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
   if (!is.null(forced))  {
     donnes <- data[,c(DV,forced)]	
     if (anyNA(donnes)) {donnes <- na.omit(donnes); NAflag = TRUE} else {NAflag = FALSE}	
-    formMAIN <- as.formula(paste(DV, paste(forced, collapse=" + "), sep=" ~ "))
   }
   
   if (!is.null(hierarchical))  {
@@ -43,20 +52,37 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
   
   
   # gathering all of the predictor variable names
-  allIVnoms <- -9999
+  allIVnoms <- c()
   if (!is.null(forced))        allIVnoms <- c(allIVnoms, forced)
-  if (!is.null(COVARS))        allIVnoms <- c(allIVnoms, COVARS)
-  if (!is.null(hierarchical))  allIVnoms <- c(allIVnoms, unlist(hierarchical) )
-  allIVnoms <- allIVnoms[-1]
-  
+  if (!is.null(hierarchical))  allIVnoms <- c(allIVnoms, unlist(hierarchical))
+
   # a version of donnes that contains only the variables in the analyses
-  donnesRED <- donnes[c(DV,allIVnoms)]  
+  donnesMOD <- donnes[c(DV,allIVnoms)]  
+  
+  # converting character variables to factors
+  for (lupe in 1:length(allIVnoms)) {
+    if (is.character(donnesMOD[,allIVnoms[lupe]])) {
+      message('\n', allIVnoms[lupe], ' is a character variable. It will be converted into a factor.\n')
+      donnesMOD[,allIVnoms[lupe]] <- as.factor(donnesMOD[,allIVnoms[lupe]])
+    }
+  }
+
+  # check if all IVs are either numeric or factors
+  IVclasses <- sapply(donnesMOD[,allIVnoms], class)
+  if (!all(IVclasses == 'numeric' | IVclasses == 'integer' | IVclasses == 'factor')) {
+    message('\n The following variables are not numeric, integers, or factors. Expect errors.')
+    names(
+    IVclasses[IVclasses != 'numeric' &  IVclasses != 'integer' & IVclasses != 'factor' ])
+  }
+
+  # check if DV is numeric
+  if (!is.numeric(donnesMOD[,DV])) message('\n DV is not numeric. Expect errors.')
   
   
   # descriptives
   if (verbose) {
     # descriptives for numeric variables
-    donnesNUM <- donnes[sapply(donnesRED,is.numeric)] # selecting only numeric variables
+    donnesNUM <- donnesMOD[sapply(donnesMOD,is.numeric)] # selecting only numeric variables
     if (ncol(donnesNUM) != 0) {
       minmax <- t(apply(donnesNUM, 2, range))
       descs <- data.frame(Mean=colMeans(donnesNUM), SD=apply(donnesNUM, 2, sd), 
@@ -66,20 +92,64 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
     }
     
     # frequencies for factors
-    donnesFAC <- donnes[sapply(donnesRED,is.factor)] # selecting only factor variables
+    donnesFAC <- donnesMOD[sapply(donnesMOD,is.factor)] # selecting only factor variables
     if (ncol(donnesFAC) != 0) {
       message('\n\nCategory frequencies for the factor variables:\n')
       print(apply((donnesFAC), 2, table))
     }
-    rm(donnesRED, donnesNUM, donnesFAC)
+    
+    rm(donnesNUM, donnesFAC)
   }
   
   
+  # test if factor variables have contrasts; if not, add contrasts with names
+  IV_factors <- IVclasses[IVclasses == 'factor' ]
+  if (length(IV_factors) > 0) {
+    for (lupe in 1:length(IV_factors)) {
+      
+      if (is.null(attr( donnesMOD[,names(IV_factors[lupe])], "contrasts" ))) {
+        
+        fac_levels <- levels(donnesMOD[,names(IV_factors[lupe])])
+        
+        # print(fac_levels)
+        # print(lupe)
+        
+        # The baseline group is based on alphabetic/numerical order, unless the terms "control"
+        # or "Control" or "baseline" or "Baseline" appear in the names of a factor
+        # level, in which case that factor level is used as the dummy codes baseline.
+        if (length(grep(pattern = 'Control|control|baseline|Baseline', x = fac_levels)) > 0) {
+          base_level <- grep(pattern = 'Control|control|baseline|Baseline', x = fac_levels)
+        } else {
+          base_level <- order(fac_levels, decreasing = FALSE, na.last = TRUE)[1]
+        }
+
+        custom_contrasts <- contr.treatment(length(fac_levels), base = base_level)
+        
+        colnames(custom_contrasts) <-
+          better_contrast_noms(contrasts(donnesMOD[,names(IV_factors[lupe])]))
+        
+        # apply the contrast matrix to the factor variable
+        contrasts(donnesMOD[,names(IV_factors[lupe])]) <- custom_contrasts
+      }
+    }
+  }
+    
+  # display the contrasts for factors
+  if (verbose & length(IV_factors) > 0) {
+    for (lupe in 1:length(IV_factors)) {
+      message('\nThe contrasts for ', names(IV_factors[lupe]), ' are:\n')
+      print(contrasts(donnesMOD[,names(IV_factors[lupe])]), print.gap=4)
+    }
+  }
   
+  
+
   # non hierarchical regression (without interactions or blocks)
   if (!is.null(forced))  {
     
-    modelMAIN <- lm(formMAIN, data=donnes, model=TRUE, x=TRUE, y=TRUE)
+    formMAIN <- as.formula(paste(DV, paste(forced, collapse=" + "), sep=" ~ "))
+    
+    modelMAIN <- lm(formMAIN, data=donnesMOD, model=TRUE, x=TRUE, y=TRUE)
     
     modelMAINsum <- summary(modelMAIN)
     
@@ -88,12 +158,12 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
     # Using the classical R^2 test statistic for (linear) regression designs, 
     # this function computes the corresponding Bayes factor test
     # the Bayes factor (against the intercept-only null)
-    Nvars <- ncol(donnes) - 1
-    BF_mod_Rsq <- linearReg.R2stat(R2 = RsqMAIN, N = nrow(donnes), p = Nvars, 
+    Nvars <- ncol(donnesMOD) - 1
+    BF_mod_Rsq <- linearReg.R2stat(R2 = RsqMAIN, N = nrow(donnesMOD), p = Nvars, 
                                    simple = TRUE)
     
     # Anova Table (Type III tests)
-    anova_table <- ANOVA_TABLE(data=donnes, model=modelMAIN) 
+    anova_table <- ANOVA_TABLE(data=donnesMOD, model=modelMAIN) 
     
     # creating a new version of the raw data, from the lm output, so that can 
     # provide stats for dummy codes
@@ -101,11 +171,9 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
     modeldata <- data.frame(modelMAIN$y, modelMAIN$x[,2:ncol(modelMAIN$x)])
     colnames(modeldata) <- c(DV,colnames(modelMAIN$x)[2:ncol(modelMAIN$x)])
     
-    # mainRcoefs <- PARTIAL_COEFS(cormat=cor(modeldata), 
-    #                             modelRsq=summary(modelMAIN)$r.squared, verbose=FALSE)
-    
     mainRcoefs <- 
-      PARTIAL_COR(data=cor(modeldata), Y=DV, X=forced, Ncases=nrow(donnes), verbose=FALSE)
+      PARTIAL_COR(data=cor(modeldata), Y=DV, X=names(modelMAIN$coefficients)[-1], 
+                  Ncases=nrow(donnesMOD), verbose=FALSE)
     
     mainRcoefs <- cbind(mainRcoefs$betas, mainRcoefs$Rx_y, 
                         mainRcoefs$R_partials, mainRcoefs$R_semipartials)
@@ -128,7 +196,7 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
     
     if (MCMC) {
       
-      mod_lmBF <- lmBF(formMAIN, data = donnes) 
+      mod_lmBF <- lmBF(formMAIN, data = donnesMOD) 
       
       # Bayes factors - model   Against denominator: Intercept only
       BF_mod_h1h0 <- unlist(extractBF(mod_lmBF)[1])
@@ -153,14 +221,14 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
       # when all uncentered covariates are equal to 0.
       
       # standardized estimates & CIs
-      SD_preds <- apply(donnes,2,sd)
+      SD_preds <- apply(donnesMOD,2,sd)
       ests_z <- c(NA, (ests[-1] * SD_preds[-1]) / SD_preds[1])
       ests_z_ci_lb <- c(NA, (ests_ci_lb[-1] * SD_preds[-1]) / SD_preds[1])
       ests_z_ci_ub <- c(NA, (ests_ci_ub[-1] * SD_preds[-1]) / SD_preds[1])
       
       # Bayes factors - for the estimates, from just t & N
       t_values <- modelMAINsum$coefficients[,3]
-      BF_ests_h1h0 <- unlist(sapply(t_values, ttest.tstat, n1 = nrow(donnes), 
+      BF_ests_h1h0 <- unlist(sapply(t_values, ttest.tstat, n1 = nrow(donnesMOD), 
                                     simple=TRUE))
       BF_ests_h0h1 <- 1 / BF_ests_h1h0
       
@@ -236,7 +304,7 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
     # add, if any, factor variables in data to modeldata 
     # (because lm changes names & types and the original variables are needed for PLOTMODEL)
     factor_variables <- names(modelMAIN$model[sapply(modelMAIN$model, is.factor)])
-    if (!is.null(factor_variables))  modeldata[factor_variables] <- donnes[,factor_variables]
+    if (!is.null(factor_variables))  modeldata[factor_variables] <- donnesMOD[,factor_variables]
     
     output <- list(modelMAIN=modelMAIN, modelMAINsum=modelMAINsum, 
                    anova_table=anova_table, mainRcoefs=mainRcoefs, 
@@ -266,7 +334,7 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
       
       if (lupe > 1) preds <- c(preds, unlist(hierarchical[lupe]))
       
-      donnesH <- donnes[,c(DV,preds)]
+      donnesH <- donnesMOD[,c(DV,preds)]
       
       formMAIN <- as.formula(paste(DV, paste(preds, collapse=" + "), sep=" ~ "))
       
@@ -291,11 +359,9 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
       modeldata <- data.frame(modelMAIN$y, modelMAIN$x[,2:ncol(modelMAIN$x)])
       colnames(modeldata) <- c(DV,colnames(modelMAIN$x)[2:ncol(modelMAIN$x)])
       
-      # mainRcoefs <- PARTIAL_COEFS(cormat=cor(modeldata), 
-      #                             modelRsq=summary(modelMAIN)$r.squared, verbose=FALSE)
-      
       mainRcoefs <- 
-        PARTIAL_COR(data=cor(modeldata), Y=DV, X=preds, Ncases=nrow(donnesH), verbose=FALSE)
+        PARTIAL_COR(data=cor(modeldata), Y=DV, X=names(modelMAIN$coefficients)[-1], 
+                    Ncases=nrow(donnesH), verbose=FALSE)
       
       mainRcoefs <- cbind(mainRcoefs$betas, mainRcoefs$Rx_y, 
                           mainRcoefs$R_partials, mainRcoefs$R_semipartials)
@@ -450,7 +516,7 @@ OLS_REGRESSION <- function (data, DV, forced=NULL, hierarchical=NULL,
     # add, if any, factor variables in data to modeldata 
     # (because lm changes names & types and the original variables are needed for PLOTMODEL)
     factor_variables <- names(modelMAIN$model[sapply(modelMAIN$model, is.factor)])
-    if (!is.null(factor_variables))  modeldata[factor_variables] <- donnes[,factor_variables]
+    if (!is.null(factor_variables))  modeldata[factor_variables] <- donnesMOD[,factor_variables]
     
     
     output <- list(modelMAIN=modelMAIN, modelMAINsum=modelMAINsum, 
